EXPORTS
	adler32@12                                   @1
	compress@16                                  @2
	crc32@12                                     @3
	deflate@8                                    @4
	deflateCopy@8                                @5
	deflateEnd@4                                 @6
	deflateInit2_@32                             @7
	deflateInit_@16                              @8
	deflateParams@12                             @9
	deflateReset@4                               @10
	deflateSetDictionary@12                      @11
	gzclose@4                                    @12
	gzdopen@8                                    @13
	gzerror@8                                    @14
	gzflush@8                                    @15
	gzopen@8                                     @16
	gzread@12                                    @17
	gzwrite@12                                   @18
	inflate@8                                    @19
	inflateEnd@4                                 @20
	inflateInit2_@16                             @21
	inflateInit_@12                              @22
	inflateReset@4                               @23
	inflateSetDictionary@12                      @24
	inflateSync@4                                @25
	uncompress@16                                @26
	zlibVersion@0                                @27
	gzprintf                                     @28
	gzputc@8                                     @29
	gzgetc@4                                     @30
	gzseek@12                                    @31
	gzrewind@4                                   @32
	gztell@4                                     @33
	gzeof@4                                      @34
	gzsetparams@12                               @35
	zError@4                                     @36
	inflateSyncPoint@4                           @37
	get_crc_table@0                              @38
	compress2@20                                 @39
	gzputs@8                                     @40
	gzgets@12                                    @41
