// RCS: @(#) $Id: img.rc,v 1.1.1.1 2002/05/10 17:20:09 andreas_kupries Exp $
//
// Version
//

#include <windows.h>
#define RESOURCE_INCLUDED
#include <img.h>

#define STRINGIFY1(x)       #x
#define STRINGIFY(x)        STRINGIFY1(x) 
 
VS_VERSION_INFO VERSIONINFO
 FILEVERSION    IMG_MAJOR_VERSION,IMG_MINOR_VERSION,IMG_RELEASE_LEVEL,IMG_RELEASE_SERIAL
 PRODUCTVERSION IMG_MAJOR_VERSION,IMG_MINOR_VERSION,IMG_RELEASE_LEVEL,IMG_RELEASE_SERIAL
 FILEFLAGSMASK	0x3fL
 FILEFLAGS	0x0L
 FILEOS		0x4L
 FILETYPE	0x2L
 FILESUBTYPE	0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904b0"
        BEGIN
            VALUE "FileDescription", "Img DLL\0"
            VALUE "OriginalFilename", "img" STRINGIFY(IMG_MAJOR_VERSION) STRINGIFY(IMG_MINOR_VERSION) ".dll\0"
            VALUE "CompanyName", "Jan Nijtmans\0"
            VALUE "FileVersion", IMG_PATCH_LEVEL
            VALUE "LegalCopyright", "Copyright \251 1995-1999\0"
            VALUE "ProductName", "Img " IMG_VERSION " for Windows\0"
            VALUE "ProductVersion", IMG_PATCH_LEVEL
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1200
    END
END

//
// Icons
//

img                      ICON    DISCARDABLE     "img.ico"
