LIBRARY libtiff
EXPORTS TIFFOpen
	TIFFGetVersion
	TIFFClose
	TIFFFlush
	TIFFFlushData
	TIFFGetField
	TIFFVGetField
	TIFFGetFieldDefaulted
	TIFFVGetFieldDefaulted
	TIFFReadDirectory
	TIFFScanlineSize
	TIFFStripSize
	TIFFVStripSize
	TIFFTileRowSize
	TIFFTileSize
	TIFFVTileSize
	TIFFFileno
	TIFFGetMode
	TIFFIsTiled
	TIFFIsByteSwapped
	TIFFCurrentRow
	TIFFCurrentDirectory
	TIFFCurrentStrip
	TIFFCurrentTile
	TIFFReadBufferSetup
	TIFFLastDirectory
	TIFFSetDirectory
	TIFFSetSubDirectory
	TIFFUnlinkDirectory
	TIFFSetField
	TIFFVSetField
	TIFFWriteDirectory
	TIFFPrintDirectory
	TIFFReadScanline
	TIFFWriteScanline
	TIFFReadRGBAImage
	TIFFPrintDirectory
	TIFFReadScanline
	TIFFWriteScanline
	TIFFReadRGBAImage
	TIFFFdOpen
	TIFFClientOpen
	TIFFFileName
	TIFFError
	TIFFWarning
	TIFFSetErrorHandler
	TIFFSetWarningHandler
	TIFFComputeTile
	TIFFCheckTile
	TIFFNumberOfTiles
	TIFFReadTile
	TIFFWriteTile
	TIFFComputeStrip
	TIFFNumberOfStrips
	TIFFReadEncodedStrip
	TIFFReadRawStrip
	TIFFReadEncodedTile
	TIFFReadRawTile
	TIFFWriteEncodedStrip
	TIFFWriteRawStrip
	TIFFWriteEncodedTile
	TIFFWriteRawTile
	TIFFSetWriteOffset
	TIFFSwabShort
	TIFFSwabLong
	TIFFSwabArrayOfShort
	TIFFSwabArrayOfLong
	TIFFReverseBits
	TIFFGetBitRevTable
	_TIFFmalloc
	_TIFFrealloc
	_TIFFfree
	_TIFFmemset
	_TIFFmemcpy
	_TIFFmemcmp
	TIFFCurrentDirOffset
	TIFFDefaultDirectory
	TIFFDefaultTileSize
	TIFFFindCODEC
	TIFFFlushData1
	TIFFFreeDirectory
	TIFFIsMSB2LSB
	TIFFIsUpSampled
	TIFFNumberOfDirectories
	TIFFPredictorInit
	TIFFRGBAImageBegin
	TIFFRGBAImageEnd
	TIFFRGBAImageGet
	TIFFRGBAImageOK
	TIFFRasterScanlineSize
	TIFFReadRGBAStrip
	TIFFReadRGBATile
	TIFFReassignTagToIgnore
	TIFFRegisterCODEC
	TIFFSetCompressionScheme
	TIFFSetTagExtender
	TIFFSwabArrayOfDouble
	TIFFSwabDouble
	TIFFUnRegisterCODEC
	TIFFWriteBufferSetup
	_TIFFDefaultStripSize
	_TIFFDefaultTileSize
	_TIFFFax3fillruns
	_TIFFFieldWithTag
	_TIFFFindFieldInfo
	_TIFFMergeFieldInfo
	_TIFFNoPostDecode
	_TIFFNoPreCode
	_TIFFNoRowDecode
	_TIFFNoRowEncode
	_TIFFNoSeek
	_TIFFNoStripDecode
	_TIFFNoStripEncode
	_TIFFNoTileDecode
	_TIFFNoTileEncode
	_TIFFPrintFieldInfo
	_TIFFSampleToTagType
	_TIFFSetDefaultCompressionState
	_TIFFSetupFieldInfo
	_TIFFSwab16BitData
	_TIFFSwab32BitData
	_TIFFSwab64BitData
	_TIFFgetMode
	_TIFFprintAscii
	_TIFFprintAsciiTag
	_TIFFsetByteArray
	_TIFFsetDoubleArray
	_TIFFsetFloatArray
	_TIFFsetLongArray
	_TIFFsetNString
	_TIFFsetShortArray
	_TIFFsetString

