/* 
 * tkimgStubInit.c --
 */

#include "tkimg.h"

/*
 * Remove macros that will interfere with the definitions below.
 */


/*
 * WARNING: The contents of this file is automatically generated by the
 * genStubs.tcl script. Any modifications to the function declarations
 * below should be made in the tkimg.decls script.
 */

/* !BEGIN!: Do not edit below this line. */

TkimgStubs tkimgStubs = {
    TCL_STUB_MAGIC,
    NULL,
    tkimg_OpenFileChannel, /* 0 */
    tkimg_ReadInit, /* 1 */
    tkimg_WriteInit, /* 2 */
    tkimg_Getc, /* 3 */
    tkimg_Read, /* 4 */
    tkimg_Putc, /* 5 */
    tkimg_Write, /* 6 */
    tkimg_ReadBuffer, /* 7 */
    NULL, /* 8 */
    NULL, /* 9 */
    tkimg_PhotoPutBlock, /* 10 */
    NULL, /* 11 */
    NULL, /* 12 */
    NULL, /* 13 */
    NULL, /* 14 */
    NULL, /* 15 */
    NULL, /* 16 */
    NULL, /* 17 */
    NULL, /* 18 */
    NULL, /* 19 */
    tkimg_FixChanMatchProc, /* 20 */
    tkimg_FixObjMatchProc, /* 21 */
    tkimg_FixStringWriteProc, /* 22 */
    NULL, /* 23 */
    NULL, /* 24 */
    NULL, /* 25 */
    NULL, /* 26 */
    NULL, /* 27 */
    NULL, /* 28 */
    NULL, /* 29 */
    tkimg_GetStringFromObj, /* 30 */
    tkimg_GetByteArrayFromObj, /* 31 */
    tkimg_ListObjGetElements, /* 32 */
};

/* !END!: Do not edit above this line. */
