EXPORTS
	adler32=adler32@12                           @1
	compress=compress@16                         @2 
	crc32=crc32@12                               @3
	deflate=deflate@8                            @4
	deflateCopy=deflateCopy@8                    @5
	deflateEnd=deflateEnd@4                      @6
	deflateInit2_=deflateInit2_@32               @7
	deflateInit_=deflateInit_@16                 @8
	deflateParams=deflateParams@12               @9
	deflateReset=deflateReset@4                  @10
	deflateSetDictionary=deflateSetDictionary@12 @11
	gzclose=gzclose@4                            @12
	gzdopen=gzdopen@8                            @13
	gzerror=gzerror@8                            @14
	gzflush=gzflush@8                            @15
	gzopen=gzopen@8                              @16
	gzread=gzread@12                             @17
	gzwrite=gzwrite@12                           @18
	inflate=inflate@8                            @19
	inflateEnd=inflateEnd@4                      @20
	inflateInit2_=inflateInit2_@16               @21
	inflateInit_=inflateInit_@12                 @22
	inflateReset=inflateReset@4                  @23
	inflateSetDictionary=inflateSetDictionary@12 @24
	inflateSync=inflateSync@4                    @25
	uncompress=uncompress@16                     @26
	zlibVersion=zlibVersion@0                    @27
	gzprintf                                     @28
	gzputc=gzputc@8                              @29
	gzgetc=gzgetc@4                              @30
	gzseek=gzseek@12                             @31
	gzrewind=gzrewind@4                          @32
	gztell=gztell@4                              @33
	gzeof=gzeof@4                                @34
	gzsetparams=gzsetparams@12                   @35
	zError=zError@4                              @36
	inflateSyncPoint=inflateSyncPoint@4          @37 
	get_crc_table=get_crc_table@0                @38
	compress2=compress2@20                       @39
	gzputs=gzputs@8                              @40
	gzgets=gzgets@12                             @41
