#ifdef WIN32
/* Restore the usage of the original names */
#undef zlibVersion
#undef zError
#undef crc32
#undef adler32
#undef deflateInit_
#undef deflateInit2_
#undef deflate
#undef deflateEnd
#undef deflateSetDictionary
#undef deflateCopy
#undef deflateReset
#undef deflateParams
#undef compress
#undef compress2
#undef inflateInit_
#undef inflateInit2_
#undef inflate
#undef inflateEnd
#undef inflateSetDictionary
#undef inflateSync
#undef inflateReset
#undef uncompress
#undef gzopen
#undef gzdopen
#undef gzsetparams
#undef gzread
#undef gzwrite
#undef gzputs
#undef gzgets
#undef gzputc
#undef gzgetc
#undef gzflush
#undef gzseek
#undef gzrewind
#undef gztell
#undef gzeof
#undef gzclose
#undef gzerror
#endif /* WIN32 */
